<?php

namespace Drupal\language_switcher_menu\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides menu links to switch to another language.
 */
class LanguageSwitcherLink extends DeriverBase implements ContainerDeriverInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Constructs a new LanguageSwitcherLink instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager, PathMatcherInterface $path_matcher) {
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('config.factory'),
      $container->get('language_manager'),
      $container->get('path.matcher'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $links = [];

    if (!$this->languageManager->isMultilingual()) {
      return $links;
    }

    $config = $this->configFactory->get('language_switcher_menu.settings');

    if (empty($config->get('type'))) {
      return $links;
    }
    if (empty($config->get('parent'))) {
      return $links;
    }

    $route_name = $this->pathMatcher->isFrontPage() ? '<front>' : '<current>';
    $link_info = $this->languageManager->getLanguageSwitchLinks($config->get('type'), Url::fromRoute($route_name));
    if (!isset($link_info->links)) {
      return $links;
    }

    $parent_config = explode(':', $config->get('parent'));
    $menu_name = $parent_config[0];
    $parent = NULL;
    if (!empty($parent_config[1])) {
      unset($parent_config[0]);
      $parent = implode(':', $parent_config);
    }

    $weight = (int) $config->get('weight');

    foreach ($link_info->links as $id => $link) {
      $links[$id] = [
        'title' => $link['title'],
        'route_name' => $link['url']->getRouteName(),
        'route_parameters' => $link['url']->getRouteParameters(),
        'menu_name' => $menu_name,
        'parent' => $parent,
        'weight' => $weight,
        'options' => [
          'language' => $link['language'],
          'set_active_class' => TRUE,
        ] + (isset($link['query']) ? [
          'query' => $link['query'],
        ] : []) + (isset($link['attributes']) ? [
          'attributes' => $link['attributes'],
        ] : []),
      ] + $base_plugin_definition;
      $weight + 1;
    }

    return $links;
  }

}
